/*!
   ZTEX Firmware Kit for EZ-USB FX3 Microcontrollers
   Copyright (C) 2009-2014 ZTEX GmbH.
   http://www.ztex.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!*/
/* 
    Template for default firmware.
*/

#undef GPIO_SIMPLE_BITMAP0
#define GPIO_SIMPLE_BITMAP0 ( ( 1 << GPIO_GPIO0 ) | ( 1 << GPIO_GPIO1 ) | ( 1 << GPIO_RESET ) | ( 1 << GPIO_CLK ) | ( 1 << GPIO_DATA ) | ( 1 << GPIO_STOP ) )
#undef GPIO_SIMPLE_BITMAP1
#define GPIO_SIMPLE_BITMAP1 ( ( 1 << (GPIO_GPIO2-32) ) | ( 1 << (GPIO_GPIO3-32) ) )

CyU3PDmaChannel dma_out_handle, dma_in_handle;
CyBool_t fpga_running = CyFalse;

#define EZUSB_IO_IN_SOCKET CY_U3P_PIB_SOCKET_1
#define EZUSB_IO_OUT_SOCKET CY_U3P_PIB_SOCKET_0

/* endpount / dma macros
    OUT_ENDPOINT : host --> EZUSB --> FPGA 
      * buffer size: super seed: 16*1024; high speed: 512
      * buffer count: super seed: 3; high speed: 64
      * PIB socket: 1
    IN_ENDPOINT : FPGA --> EZUSB --> host
      * buffer size: super seed: 16*1024; high speed: 512
      * buffer count: super seed: 3; high speed: 64
      * PIB socket: 0
*/   

#undef EP_SETUP
#define EP_SETUP \
    INTERFACE(0, \
	EP_BULK(OUT_ENDPOINT, OUT, 16, \
	    DMA(dma_in_handle, CY_U3P_DMA_TYPE_AUTO, IS_SUPER_SPEED ? 3 : 64 , EZUSB_IO_IN_SOCKET, \
		CB(0,0) \
	    ) \
	) \
	EP_BULK(IN_ENDPOINT, IN, 16, \
	    DMA(dma_out_handle, CY_U3P_DMA_TYPE_AUTO, IS_SUPER_SPEED ? 3 : 64 , EZUSB_IO_OUT_SOCKET, ) \
	) \
    )

#include "ztex-lsi.c"
#include "ztex.c" 	
#include "ztex-ezusb-io1.c"

void run () {
    uint16_t snd_errors, rcv_errors;
    uint16_t last_snd_errors = 0;
    uint16_t last_rcv_errors = 0;
    
    ztex_log ( "Info: Starting default Firmware" );

    while (1) {
	snd_errors = ZTEX_USB3_SND_ERROR_COUNT;
	rcv_errors = ZTEX_USB3_RCV_ERROR_COUNT;
	if ( (snd_errors != last_snd_errors) || (rcv_errors != last_rcv_errors) ) {
    	    ZTEX_LOG("snd errors: %d,  rcv errors: %d", snd_errors, rcv_errors);
    	    last_snd_errors = snd_errors;
    	    last_rcv_errors = rcv_errors;
        }
        CyU3PThreadSleep (500);
    }
    
}

void usb_start() {
    // start communication if cable is connected and FPGA is running
    if ( ZTEX_FPGA_CONFIGURED ) {
	ztex_log ( "Info: Starting communication" );
	
	// start lsi, reset is active
	ztex_lsi_start();

	// load GPIF
	ztex_ezusb_io1_start( EZUSB_IO_IN_SOCKET, EZUSB_IO_OUT_SOCKET );

	// start transfers
	ZTEX_REC(CyU3PDmaChannelSetXfer (&dma_in_handle, 0));
	ZTEX_REC(CyU3PDmaChannelSetXfer (&dma_out_handle, 0));

	ztex_gpio_set(GPIO_RESET, CyFalse); 
	ztex_gpio_set(ZTEX_GPIO_LED, CyTrue);
    }
}

void usb_stop() {
    if ( ZTEX_FPGA_CONFIGURED ) {
	ztex_log ( "Info: Stopping communication" );
	
	ztex_ezusb_io1_stop();
	
	ztex_lsi_stop();

	ztex_gpio_set(ZTEX_GPIO_LED, CyFalse);
    }
}

/*
 * Main function
 */
int main (void)
{
    ztex_pib_clock.clkDiv = 4;		// normal setting: 104 MHz @ 26 MHz external clock
//  ztex_pib_clock.clkDiv = 6;		// conservative setting: 69.33MHz @ 26 MHz external clock

    ztex_app_thread_run = run;
    ztex_usb_start = usb_start;
    ztex_usb_stop = usb_stop;
    
    ztex_main();  	// startes the OS and never returns
    return 0;		// makes the compiler happy
}
